# SloBot release 6 by David Bowland
# ./addons/eventscripts/slobot/slobot.py

# >>> To configure this addon please see slobot.cfg <<<

"""
Adjusts the speed of bots based on the number of remaining team members, allows players to walk through bots, and removes certain door types to facilitate bot movement. Requires only ES 2.0+
"""


import es
import gamethread
import math
import os.path
import random

import psyco
psyco.full()


info = es.AddonInfo()
info.name     = 'SloBot'
info.version  = '6'
info.url      = 'http://addons.eventscripts.com/addons/view/slobot'
info.basename = 'slobot'
info.author   = 'SuperDave'


###


class NoBlockManager:
   userid  = 0
   noblock = False

   def __init__(self, userid):
      self.userid = userid

   def playerSpawn(self):
      if int(es.ServerVar('slobot_noblock')) > 1:
         self._noBlock(True)
         self.noblock = True

   def removeNoBlock(self):
      if self.noblock:
         self._noBlock(False)
         self.noblock = False

   def enterCombat(self):
      pass

   def bombPlanted(self):
      pass

   def removeDelay(self):
      pass

   def _noBlock(self, state):
      es.setplayerprop(self.userid, 'CBaseEntity.m_CollisionGroup', 2 if state else 5)


class SloBotManager(NoBlockManager):
   in_combat = False

   def playerSpawn(self):
      if int(es.ServerVar('slobot_noblock')):
         self._noBlock(True)
         self.noblock = True

      self._setNewSpeed()

   def enterCombat(self):
      self.removeDelay()
      self.in_combat = True

      self.__setSpeed(1)

      gamethread.delayedname(es.ServerVar('slobot_speed_unrestricteddelay'), 'slobot_%s' % self.userid, self._exitCombat)

   def _exitCombat(self):
      self._setNewSpeed()

      self.in_combat = False

   def bombPlanted(self):
      self.removeDelay()
      self._setNewSpeed()

   def removeDelay(self):
      if self.in_combat:
         gamethread.cancelDelayed('slobot_%s' % self.userid)
         self.in_combat = False

   def resetSpeed(self):
      if self.in_combat:
         self.removeDelay()
         self.__setSpeed(1)

   def _setNewSpeed(self):
      speed_start, speed_base = ('slobot_speed_', 5) if es.getentityindex('planted_c4') == -1 else ('slobot_speed_bombplanted_', 10)

      speed_range              = range(es.ServerVar(speed_start + 'min'), es.ServerVar(speed_start + 'max') + 1)
      remaining_player_percent = float(es.ServerVar('slobot_speed_playersleft_percent'))

      team       = es.getplayerteam(self.userid)
      team_count = es.getplayercount(team)

      if len(speed_range) > 1 and remaining_player_percent:
         float_percent            = 100.
         remaining_player_percent = 100. - (remaining_player_percent * (team_count - es.getlivingplayercount(team)) / team_count)

         new_range = []
         for x in range(len(speed_range)):
            new_range += [speed_range[x]] * int(math.ceil((remaining_player_percent / 100.) ** x * len(speed_range)))

         speed_range[:] = new_range

      self.__setSpeed((float(random.choice(speed_range) + speed_base) / 10.) if speed_range else 1)

   def __setSpeed(self, speed):
      es.setplayerprop(self.userid, 'CBasePlayer.localdata.m_flLaggedMovementValue', speed)


players = {}


def getPlayer(userid):
   global players

   userid = int(userid)
   if not players.has_key(userid):
      players[userid] = SloBotManager(userid) if es.getplayersteamid(userid) == 'BOT' else NoBlockManager(userid)

   return players[userid]


def removePlayer(userid):
   global players

   userid = int(userid)
   if players.has_key(userid):
      players[userid].removeDelay()

      del players[userid]


###


dict_options = {'slobot_speed_unrestricteddelay':[7, 'Number of seconds bots have unrestricted movement after combat'], 'slobot_speed_min':[0, 'Minimum speed a bot will receive during normal play--0 (50% normal speed) to slobot_speed_max'], 'slobot_speed_max':[5, 'Maximum speed a bot will receive during normal play--slobot_speed_min to 5 (normal speed)'], 'slobot_speed_bombplanted_min':[0, 'Minimum speed a bot will receive after the bomb is planted--0 (normal speed) to slobot_speed_bombplanted_max'], 'slobot_speed_bombplanted_max':[0, 'Maximum speed a bot will receive after the bomb is planted--slobot_speed_bombplanted_min to 5 (150% normal speed)'], 'slobot_speed_playersleft_percent':[60,'Percent of bot speed that is affected by the number of live teammates'], 'slobot_removerotatingdoors':[0, '0 = no change, 1 = certain types of movable doors are removed from maps'], 'slobot_noblock':[1, '0 = no change, 1 = players can walk through bots, 2 = players can walk through all other players']}


def load():
   """
   Checks to make sure critical server variables are created by the config
   Loops through each player calling player_spawn
   Calls remove_doors
   """
   for str_option in dict_options:
      es.ServerVar(str_option, dict_options[str_option][0], dict_options[str_option][1])
   if os.path.isfile(es.ServerVar('eventscripts_addondir') + '/slobot/slobot.cfg'):
      es.server.cmd('es_xmexec ../addons/eventscripts/slobot/slobot.cfg')
   else:
      es.dbgmsg(0, 'SloBot: Unable to load slobot.cfg! Please ensure it is in the ./slobot/ directory.')

   for userid in filter(lambda x: not es.getplayerprop(x, 'CBasePlayer.pl.deadflag'), es.getUseridList()):
      getPlayer(userid).playerSpawn()

   remove_doors()


def es_map_start(event_var):
   for userid in players:
      players[userid].removeDelay()

   players.clear()


def round_start(event_var):
   remove_doors()


def player_spawn(event_var):
   getPlayer(event_var['userid']).playerSpawn()


def player_blind(event_var):
   getPlayer(event_var['userid']).enterCombat()


def player_hurt(event_var):
   if event_var['es_userteam'] <> event_var['es_attackerteam']:
      getPlayer(event_var['userid']).enterCombat()

      attacker = int(event_var['attacker'])
      if attacker:
         getPlayer(attacker).enterCombat()


def player_death(event_var):
   getPlayer(event_var['userid']).removeDelay()

   attacker = int(event_var['attacker'])
   if event_var['es_userteam'] <> event_var['es_attackerteam'] and attacker:
      getPlayer(attacker).enterCombat()


def player_disconnect(event_var):
   removePlayer(event_var['userid'])


def bomb_planted(event_var):
   for userid in players:
      players[userid].bombPlanted()


def unload():
   for userid in players:
      players[userid].resetSpeed()
      players[userid].removeNoBlock()

   for str_option in dict_options:
      es.ServerVar(str_option).set(0)


def remove_doors():
   """
   Removes doors if necessary 
   """
   if int(es.ServerVar('slobot_removerotatingdoors')):
      userid = es.getuserid()
      if not userid: return

      es.server.queuecmd('es_xfire %s func_door_rotating kill' % userid)
      es.server.queuecmd('es_xfire %s func_door kill' % userid)
      es.server.queuecmd('es_xfire %s prop_door_rotating kill' % userid)